'''
Script to
Author: Steffen Schotthoefer
Version: 0.0
Date 22.10.2021
'''

from src.solver import MNSolver1D
from src.utils import plot_1d
import numpy as np


def main():
    print("---------- Start hybrid solver demonstration (single threaded) ------------")
    print(
        "Attention: A low gridsize is chosen for speed of execution. This implies low spatial accuracy of the solution.")
    print(
        "Please consult the appendix of the paper for results with large grids")

    # --- M1 1D ---

    nx = 100
    order = 1
    t_f = 0.7

    solver_icnn = MNSolver1D.MNSolver1D(traditional=False, polyDegree=order, model_mk=11, gridsize=nx)
    _, u_icnn = solver_icnn.solve(max_iter=100000, t_end=t_f)

    solver_mono = MNSolver1D.MNSolver1D(traditional=False, polyDegree=order, model_mk=15, gridsize=nx)
    _, u_mono = solver_mono.solve(max_iter=100000, t_end=t_f)

    solver_newton = MNSolver1D.MNSolver1D(traditional=True, polyDegree=order, model_mk=15, gridsize=nx)
    u_newton, _ = solver_newton.solve(max_iter=100000, t_end=t_f)

    # plot results

    x = np.linspace(0, 1, nx)
    res_list = [np.transpose(u_newton), np.transpose(u_icnn), np.transpose(u_mono)]
    plot_1d([x], res_list,
            labels=[r"$u_0$", r"$u_1$", r"Convex - $u_{0,\theta}$", r"Convex - $u_{1,\theta}$",
                    r"Mono - $u_{0,\theta}$", r"Mono - $u_{1,\theta}$"],
            name="inflow_1D_M1",
            folder_name="", linetypes=['-', 'o', '^'], xlim=[0, 1], ylim=[0, 0.8],
            xlabel='x', ylabel='u', log=False, title=r"1D M1 inflow test case")

    # --- M2 1D ---

    nx = 100
    order = 2
    t_f = 0.7

    solver_icnn = MNSolver1D.MNSolver1D(traditional=False, polyDegree=order, model_mk=11, gridsize=nx)
    _, u_icnn = solver_icnn.solve(max_iter=100000, t_end=t_f)

    solver_mono = MNSolver1D.MNSolver1D(traditional=False, polyDegree=order, model_mk=15, gridsize=nx)
    _, u_mono = solver_mono.solve(max_iter=100000, t_end=t_f)

    solver_newton = MNSolver1D.MNSolver1D(traditional=True, polyDegree=order, model_mk=15, gridsize=nx)
    u_newton, _ = solver_newton.solve(max_iter=100000, t_end=t_f)

    # plot results
    x = np.linspace(0, 1, nx)
    res_list = [np.transpose(u_newton), np.transpose(u_icnn), np.transpose(u_mono)]
    plot_1d([x], res_list,
            labels=[r"$u_0$", r"$u_1$", r"$u_2$", r"Convex - $u_{0,\theta}$", r"Convex - $u_{1,\theta}$",
                    r"Convex - $u_{2,\theta}$", r"Mono - $u_{0,\theta}$", r"Mono - $u_{1,\theta}$",
                    r"Mono - $u_{2,\theta}$"],
            name="inflow_1D_M2",
            folder_name="", linetypes=['-', 'o', '^'], xlim=[0, 1], ylim=[0, 0.8],
            xlabel='x', ylabel='u', log=False, title=r"1D M2 inflow test case")

    print("Solutions computed and plotted.")
    return True


if __name__ == '__main__':
    main()
